<?php

namespace PME\Services;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Zeedhi\Framework\Session\Session;
use PME\Auth\Util\Utilities;

class ConnectionFactory extends \Zeedhi\Framework\ORM\ConnectionFactory {

    public static function updateConnectionParams(array $connectionParams, $config = null, $eventManager = null, Session $session = null){
        $nrOrg = false;
        if($session) {
            $nrOrg = $session->has('nrorg') ? $session->get('nrorg') : $session->get('NRORG');
            self::setCurrentDBName($connectionParams, $session);
        }
        if ($nrOrg && isset($connectionParams['useVpd']) && $connectionParams['useVpd'] == 'true') {
            $connectionParams = self::factoryVPDConnectionParams($connectionParams, $nrOrg);
        }

        if(empty($connectionParams["password"])) {
            $connectionParams["password"] = '';
        } else if(isset($connectionParams["isEncrypted"]) && $connectionParams["isEncrypted"] == 'true') {
            $cryptKey = isset($connectionParams["cryptKey"]) ? $connectionParams["cryptKey"] : null;
            $connectionParams["password"] = Utilities::zeedhiDecrypt($connectionParams["password"], $cryptKey);
            $connectionParams["isEncrypted"] = false;
        }
        
        $connection = DriverManager::getConnection($connectionParams, $config, $eventManager);
        self::insertSQLWebTunning($connection, $session);
        return $connection;
    }

    protected static function setCurrentDBName($connectionParams, $session) {
        //Set current DBNAME in session for use in login session
        if(isset($connectionParams['sessionMode'])) {
            $encryptedDBName = Utilities::zeedhiEncrypt($connectionParams['dbname']);
            $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedDBName));
        } else {
            $encryptedUser = Utilities::zeedhiEncrypt($connectionParams['user']);
            $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '',$encryptedUser));
        }
    }

    protected static function factoryVPDConnectionParams($connectionParams, $nrOrg) {

        if (isset($connectionParams['vpdWithWallet']) && $connectionParams['vpdWithWallet'] == 'true') {
            $connectionParams['user']        = '/';
            $connectionParams['password']    = '';
            $connectionParams['dbname']      = 'USR_ORG_' . $nrOrg;
            $connectionParams['sessionMode'] = -2147483648;

            unset($connectionParams['host']);
            unset($connectionParams['port']);
        } else {
            $connectionParams['user'] = 'USR_ORG_' . $nrOrg;
            $connectionParams['password'] = $connectionParams['vpdPassword'];
            if (isset($connectionParams['isWallet']) && $connectionParams['isWallet'] == 'true') {
                $connectionParams['dbname']  = $connectionParams['vpdDBNAME'];
                $connectionParams['host']    = $connectionParams['vpdHost'];
                $connectionParams['port']    = $connectionParams['vpdPort'];
                $connectionParams['service'] = $connectionParams['vpdService'];
                unset($connectionParams['sessionMode']);
            }
        }
        return $connectionParams;
    }

    /**
     * @param Connection $connection
     * @param Session $session
     * @throws \Exception
     */
    public static function insertSQLWebTunning($connection, $session) {
        try {
            if(Utilities::getUseSQLWebTunningParameter()) {
                $query = "INSERT INTO SQLWEBTUNNING (DSREMOTEADDR, DSREMOTEHOST, DSREMOTEAPPL, DSSERVERRDM,
                                                     DSSERVERDLL,  DTSTARTDATE,  DSREMOTEOPER, DSSERVMETHOD)
                               VALUES (:DSREMOTEADDR, :DSREMOTEHOST, :DSREMOTEAPPL, :DSSERVERRDM,
                                       :DSSERVERDLL,  SYSDATE,       :DSREMOTEOPER, :DSSERVMETHOD)";

                $host       = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];
                $requestUri = explode('index.php', trim($_SERVER['REQUEST_URI']));
                $route      = array_pop($requestUri);
                $cdoperador = $session && $session->has('CDOPERADOR') ? $session->has('CDOPERADOR') : 'SESSAO_N_INI';
                $hostArr    = explode('.', $host);
                $hostInit   = array_shift($hostArr);
                
                $params = array (
                    'DSREMOTEADDR' => trim(substr(gethostbyname($host), 0, 15)),
                    'DSREMOTEHOST' => trim(substr($hostInit, 0, 15)),
                    'DSREMOTEAPPL' => strval(Utilities::getProjectIdParameter()),
                    'DSSERVERRDM'  => trim(implode('.', $hostArr)),
                    'DSSERVERDLL'  => 'ZEEDHI_REQUEST',
                    'DSREMOTEOPER' => $cdoperador,
                    'DSSERVMETHOD' => substr($route, 0, 30)
                );
                $connection->executeQuery($query, $params);
            }
        } catch(\Exception $e) {}
    }
}