import {
  Metadata, Mask, Config, Loader, IDictionary, Http, IEventParam, FormatterParserProvider, Router,
} from '@zeedhi/core';
import {
  moment, AlertService, Modal, Grid, Select, IGridEvent, Button, Progress, TextInput, DialogService, Date as ZdDate, Text,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Utilities {
  private modal!: Modal;

  private gridPeriodo!: Grid;

  public appController = Loader.getInstance('AppController');

  // Variáveis do modal de cardápio
  private cdtipoCard = '';

  private paramsCard?: IDictionary;

  public dtIniCard = '';

  public dtFinCard = '';
  
  public dtIniPeriodo = '';

  public dtFinPeriodo = '';

  public descCard = '';
  
  // Seta o valor do filtro do PRATO de acordo com o name do component passado como parametro.
  public setFilterPrato(cdfilial: string, nmprato: string) {
    const prato = Metadata.getInstance<Select>(nmprato);
    prato.setValue(null);
    prato.datasource.filter = {
      CDFILIAL: cdfilial,
    };
  }
  
  // Verifica se o prato inicial é maior que o prato final.
  public returnPrat(pratoIni: string, pratoFin: string): boolean {
    if (pratoIni > pratoFin) {
      return true;
    }
    
    return false;
  }

  public getProcessDate(cdfilial: string) {
    const row = (cdfilial === '' || cdfilial === undefined) ? [this.appController.cdCentral] : [cdfilial];
    return Http.post('/dataProcessamento', { requestType: 'Row', row }).then((response) => { // eslint-disable-line
      return this.getBloqDate().then((bloqDate) => { // eslint-disable-line
        const processDate = response.data.dataset.processDate[0];
        const format = 'DD/MM/YYYY';
        return moment(processDate, format).add(bloqDate, 'd').format(format);
      });
    });
  }
  
  private getBloqDate() {
    return Http.get('/paramgeral').then((paramgeral) => {
      const bloqDate = paramgeral.data.data[0].QTDIABLOQCAR;
      return bloqDate;
    });
  }
  
  // seta o valor de um componente ZdDate como sendo a data de processamento
  public setDataProcessamentoZdDate({ component }: IEventParam<ZdDate>) {
    this.getProcessDate('').then((processDate: string) => {
      const dtFormatLocale = moment(processDate, 'DD/MM/YYYY').format(this.getFormatLocale());
      const dtFormatVal = moment(dtFormatLocale, this.getFormatLocale()).format(component.displayFormat);
      component.value = dtFormatVal;
    });
  }
  
  public getParamPlaneja(cdfilial?: string) {
    const filter: IDictionary = [];
    const filial = (cdfilial === '' || cdfilial === undefined) ? [this.appController.cdCentral] : [cdfilial];
    this.onMountedFilterFind(filter, 'CDFILIAL', '=', filial);
    return Http.post('/find/paramplaneja', { requestType: 'FilterData', filter }).then((response) => {
      const paramplaneja = response.data.data[0];
      return paramplaneja;
    });
  }
  
  public getCurrencySymbol(locale: string, currency: string) {
    return (0).toLocaleString(
      locale,
      {
        style: 'currency',
        currency,
        minimumFractionDigits: 0,
        maximumFractionDigits: 0,
      },
    ).replace(/\d/g, '').trim().concat(' ');
  }
  
  public getParamFilial(cdfilial: string) {
    const filter: IDictionary = [];
    this.onMountedFilterFind(filter, 'CDFILIAL', '=', (cdfilial === '' || cdfilial === undefined) ? this.appController.cdCentral : cdfilial);
    return Http.post('/find/paramfilial', { requestType: 'FilterData', filter }).then((response) => {
      const paramfilial = response.data.data[0];
      return paramfilial;
    });
  }
  
  // Exibe o alerta no topo da tela.
  public showAlert(name: string, text: string, color = 'green') {
    const alertType = {
      name,
      color,
      text,
      bottom: false,
      top: true,
      left: false,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
    AlertService.show(alertType);
  }

  public formatReportMask(mask: string) {
    return mask.replace(/[^(/\\.|\\)]/g, '@');
  }

  public getReportMaskNivel(mask: string, nivel: number): string {
    const arrayMask = mask.split('.');
    const newArrayMask = arrayMask.splice(0, nivel);
    const maskNivel = newArrayMask.join('.');
    return this.formatReportMask(maskNivel);
  }

  public timeIsValid(hour: string) {
    return /^([0-1]?[0-9]|2[0-4]):([0-5][0-9])(:[0-5][0-9])?$/.test(hour);
  }
  
  public dateIsValid(dtIni: string, dtFin: string) {
    const format = 'DD/MM/YYYY';
    const dataIni = moment(dtIni, format);
    const dataFin = moment(dtFin, format);
    if (dataIni <= dataFin) return true;
    return false;
  }

  public prodIsValid(prodIni: string, prodFin: string) {
    const productMask: string = (Config.masks.productMask as PmeMask).pattern;
    const pinicial = Mask.getValueWithMask(prodIni, productMask);
    const pfinal = Mask.getValueWithMask(prodFin, productMask);
    
    const nvinicial = pinicial.length;
    const nvfinal = pfinal.length;
    let menornv;
    if (nvinicial > nvfinal) menornv = nvfinal;
    else menornv = nvinicial;

    let resultado = true;
    let i = 0;

    while (pinicial[i] === pfinal[i] && i < menornv) i += 1;

    if ((Object.prototype.hasOwnProperty.call(pinicial, i) && !Object.prototype.hasOwnProperty.call(pfinal, i)) || (pinicial[i] > pfinal[i])) resultado = false;
    else resultado = true;

    return resultado;
  }

  public calcTamacNivel(nivel: number, tipoMask = 'produto') {
    const tpMask: string = tipoMask === 'prato' ? (Config.masks.plateMask as PmeMask).pattern : (Config.masks.productMask as PmeMask).pattern;
    const mask = tpMask.split('.');
    let tamacnivel = 0;

    for (let i = 0; i < nivel; i += 1) {
      tamacnivel += mask[i].length;
    }
    return tamacnivel;
  }

  /**
   *  Funções do gridPeriodo.json
   */
  public setCdTipoCard(cdtipoCard: string) {
    this.cdtipoCard = cdtipoCard;
  }

  public setFilterModalPeriodo() {
    if (this.gridPeriodo !== undefined) {
      const conditions = [
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'P_CDTIPOCARD',
          operator: '=',
          value: this.cdtipoCard,
        },
      ];

      this.gridPeriodo.datasource.filter = conditions;
      this.gridPeriodo.datasource.get();
    }
  }

  public setGridInstance({ component }: IGridEvent) {
    this.gridPeriodo = component;
  }

  public setPeriodoParams({ row }: IGridEvent) {
    this.paramsCard = row;
    this.dtIniCard = row.DTINICARD;
    this.dtFinCard = row.DTFINCARD;
    this.dtIniPeriodo = row.DTINICARD;
    this.dtFinPeriodo = row.DTFINCARD;
    this.descCard = row.DSCARDPME;
    const modal = this.gridPeriodo.parent as Modal;
    modal.hide();
  }

  public clearPeriodoParams() {
    this.dtIniCard = '';
    this.dtFinCard = '';
    this.dtIniPeriodo = '';
    this.dtFinPeriodo = '';
    this.descCard = '';
  }

  public getParamsCard() {
    return this.paramsCard;
  }
  
  public maskProd() {
    return (Config.masks.productMask as PmeMask).pattern;
  }
  
  public maskPlate() {
    return (Config.masks.plateMask as PmeMask).pattern;
  }
  
  public onMountedFilterFind(arr: IDictionary, nameColumn: string, operatorFind: string, valueColumn: string|number|null|undefined|IDictionary) {
    const object = {
      name: nameColumn,
      operator: operatorFind,
      value: valueColumn,
    };
    arr.push(object);
  }

  public setFilterProducts({ component }: IEventParam<Select>) {
    const nivelMax = (Config.masks.productMask as PmeMask).nivelMax;
    const conditions = [
      {
        name: 'MAXCDNVPRODUTO',
        operator: '=',
        value: nivelMax,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setDataText({ component }: IEventParam<Select>) {
    component.dataText = [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ];
  }

  public setDataTextPrato({ component }: IEventParam<Select>) {
    component.dataText = [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ];
  }

  public get setComponentPropsPrato() {
    return {
      component: 'ZdTextInput',
      mask: (Config.masks.plateMask as PmeMask).pattern,
    };
  }

  public get setComponentPropsProduto() {
    return {
      component: 'ZdTextInput',
      mask: (Config.masks.productMask as PmeMask).pattern,
    };
  }

  public pratoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    formatterDataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    dataValueOut: [
      {
        column: 'CDPRATO',
        columnOnGrid: 'CDPRATO',
      },
      {
        column: 'NMPRATO',
        columnOnGrid: 'NMPRATO',
      },
    ],
    dataValue: 'CDPRATO',
  };

  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
  };
  
  public getFormatLocaleWithWeek(): string {
    return (localStorage.getItem('LANGUAGE') !== 'en_us') ? 'DD/MM ddd' : 'MM/DD ddd';
  }
  
  public getFormatLocaleFullDataWithWeek(): string {
    return (localStorage.getItem('LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY ddd' : 'MM/DD/YYYY ddd';
  }
  
  public getFormatLocale(): string {
    return (localStorage.getItem('LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY' : 'MM/DD/YYYY';
  }

  public activeLoadingButton(button: Button) {
    const element = document.getElementsByName(button.name)[0] as HTMLElement;
    const progressEle = document.querySelectorAll('.loading-progress-line');
    progressEle.forEach((htmlElement: IDictionary) => {
      htmlElement.style.width = `${element.offsetWidth - 1}px`;
      htmlElement.style.left = `${element.offsetLeft}px`;
      htmlElement.style.top = `${element.offsetTop + (element.offsetHeight * 0.80)}px`;
      htmlElement.style.display = 'flex';
    });
    // const progressEle = document.getElementsByName(`${button.name}ProgressLine`)[0] as HTMLElement;
    button.disabled = true;
    button.cssClass = 'loadingButton';
  }

  public disableLoadingButton(button: Button) {
    const progressEle = document.querySelectorAll('.loading-progress-line');
    progressEle.forEach((htmlElement: IDictionary) => {
      htmlElement.style.display = 'none';
    });
    button.disabled = false;
    button.cssClass = '';
  }

  public centralIsValid() {
    const cdcentral = localStorage.getItem('CDFILIAL');
    let isValid = true;
    if (!cdcentral) {
      isValid = false;
      DialogService.show(
        {
          name: 'paramCentral',
          type: 'warning',
          title: 'pme::atencao',
          text: 'pme::parametrizacaoCentral',
          buttons: [
            {
              name: 'dialogButton', label: 'pme::OK', component: 'ZdButton', events: { click: () => DialogService.hide() },
            },
          ],
        },
      );
    }
    return isValid;
  }

  /**
   * Nome da barra de progresso deve seguir o formato : {nomeButton}Progress
   */
  public activeLoadingRoundButton(button: Button) {
    const progress = Metadata.getInstance<Progress>(`${button.name}Progress`);
    progress.cssClass = 'loading-progress-round';
    button.disabled = true;
  }

  public disableLoadingRoundButton(button: Button) {
    const progress = Metadata.getInstance<Progress>(`${button.name}Progress`);
    progress.cssClass = '';
    button.disabled = false;
  }

  public returnGrid(component: IDictionary) {
    let grid: IDictionary;
    if (component.type === 'button') grid = component.parent as IDictionary;
    else grid = component;
    return grid;
  }
  
  public validateNegative({ component }: IEventParam<TextInput>) {
    if (component.value < 0) component.value = '0';
  }
  
  public focusOnField(name: string) {
    const field = (document.getElementsByName(name)[0]) as HTMLElement;
    if (field) field.focus();
  }
  
  public formatValues(value: number, decimal: number): string {
    const numberMask = {
      decimalCharacter: ',',
      decimalCharacterAlternative: '.',
      digitGroupSeparator: '.',
      decimalPlaces: decimal,
    };
    const fn = FormatterParserProvider.getFormatter('ZdNumber');
    return fn(value, { mask: numberMask });
  }
  
  public removeDuplicateData(arr: IDictionary): IDictionary {
    return arr.filter((idx: number, i: number) => arr.indexOf(idx) === i);
  }
  
  public increamentZeroInHex(color: string) {
    let hexColor = color;
    for (let i = 0; i < (6 - color.length); i += 1) {
      hexColor += '0';
    }
    return hexColor;
  }
  
  public hexToRgb(hex: string) {
    const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
      r: parseInt(result[1], 16),
      g: parseInt(result[2], 16),
      b: parseInt(result[3], 16),
    } : null;
  }
  
  public RGBToTColor(rgb: IDictionary) {
    if (rgb !== null) return `rgb(${rgb.r}, ${rgb.g}, ${rgb.b})`;
    return '';
  }
  
  public TColorToHex(TColor: string) {
	  const rgb = this.TColorToRGB(TColor); // eslint-disable-line
	  return this.rgbToHex(rgb); // eslint-disable-line
  }

  public TColorToRGB(TColor: any) { // eslint-disable-line
    const r = TColor & 0xFF; // eslint-disable-line
    const g = (TColor >> 8) & 0xFF; // eslint-disable-line
    const b = (TColor >> 16) & 0xFF; // eslint-disable-line
    return [r, g, b]; // eslint-disable-line
  }

  public rgbToHex(rgb: IDictionary) {
    return '#' + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1); // eslint-disable-line
  }
  
  // define a mascara de prato de acordo com o valor/padrao
  // a logica é percorrer o padrao até inserir todos os valores: podendo ser "." ou qualquer dígito "#"
  public getMaskPrato(value: string) {
    const maskPattern: string = (Config.masks.plateMask as PmeMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}N`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }
  
  // define a mascara de produto de acordo com o valor/padrao
  // a logica é percorrer o padrao até inserir todos os valores: podendo ser "." ou qualquer dígito "#"
  public getMaskProduto(value: string) {
    const maskPattern: string = (Config.masks.productMask as PmeMask).pattern;
    const totalValue = value.length;
    let maskReturn = '';
    
    for (let i = 0, insertedNumbers = 0; insertedNumbers !== totalValue; i += 1) {
      if (maskPattern.charAt(i) === '.') {
        maskReturn = `${maskReturn}.`;
      } else {
        maskReturn = `${maskReturn}N`;
        insertedNumbers += 1;
      }
    }
    return maskReturn;
  }

  public setDtProcessFooter() {
    const currentRoute = Router.getCurrentRoute() as IDictionary;
    if (currentRoute.name !== 'login') {
      this.getProcessDate('').then((processDate: string) => {
        Metadata.getInstance<Text>('processDate').text = `${moment(processDate, 'DD/MM/YYYY').format(this.getFormatLocale())}`;
      });
    }
  }
}
